/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

////////////////////////////////////////////////////////////////////////////
//
// Module Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <stdio.h>
#include <strings.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"

extern int	nextComplexAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

////////////////////////////////////////////////////////////////////////////
//
// Module Constructor
//
////////////////////////////////////////////////////////////////////////////

Module::Module(int modId)
{
    ComplexAssembly*  ca;

    if (debugMode) {
        printf("Module::Module(modId = %d)\n", modId);
    }
    // initialize the direct type (ONTOS requirement.)
    initDirectType(MODType);     

    // initialize the simple stuff
    id = modId;
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinModuleDate + 
	(int) (random() % (MaxModuleDate-MinModuleDate+1));

    // Build a manual.
    manualReference.Reset(new (oo7Group, MANType)
    Manual(id, this), this);

    // put the manual in the database
    man()->putObject();

    // get a new set to hold all base assemblies
    allBasesReference.Reset(new (oo7Group, OC_set) Set(BAType),this);

    // now create the assemblies for the module
    if (NumAssmLevels > 1) {
        int assmId = nextComplexAssemblyId++;
        ca = new (oo7Group, CAType) ComplexAssembly (assmId, this, NULL, 1);
        designRootReference.Reset(ca);
    }
    // put the design root complex assembly to the db.
    designRoot()->putObject();

    // put the set of all bases to db
    allBases()->putObject();

}

// Additional constructor required by Ontos.
Module::Module(APL *theAPL) : Object(theAPL) {}

////////////////////////////////////////////////////////////////////////////
//
// getDirectType required by ONTOS.
//
////////////////////////////////////////////////////////////////////////////

Type* Module::getDirectType() {
    return MODType;
}


////////////////////////////////////////////////////////////////////////////
//
// Module Destructor
//
////////////////////////////////////////////////////////////////////////////

Module::~Module()
{
    if (debugMode) {
	printf("Module::~Module(id = %d)\n", id);
   }

   Destroy (FALSE);
}


////////////////////////////////////////////////////////////////////////////
//
// Module::Destroy
//
////////////////////////////////////////////////////////////////////////////

void Module::Destroy(OC_Boolean aborted)
{
    if (aborted)
	Object::Destroy (aborted);
}


////////////////////////////////////////////////////////////////////////////
//
// Module::deleteObject
//
////////////////////////////////////////////////////////////////////////////

void Module::deleteObject(OC_Boolean deallocate)
{
   // remove module from extent of all modules
   AllModulesByName->Remove (this);
   AllModulesByName->putObject();

   Object::deleteObject (deallocate);
}


////////////////////////////////////////////////////////////////////////////
//
// Module::scanManual() returns number of occurrences of the 
// character 'I' in the module's manual.
//
////////////////////////////////////////////////////////////////////////////

int Module::scanManual() 
{

  Manual *manH = man();

  if (manH == NULL) {
      fprintf(stderr, "scanManual: manH was null.\n");
      return -1;
  } else {
    return (manH->searchText('I'));
  }
}

////////////////////////////////////////////////////////////////////////////
//
// Module::firstLast() returns 1 if the first and last characters
// in the manual are the same, zero otherwise.
//
////////////////////////////////////////////////////////////////////////////

int Module::firstLast() 
{
  return (man()->firstLast());
}




/*  $Version: Module.C 1.4 93/02/25 10:08:45 Exp $ */

