/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>

#include "oo7.h"
#include <QueryIterator.h>
#include <Directory.h>
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"
#include "VarParams.h"


//////////////////////////////////////////////////////////////////
//
// Query #1 - randomly choose "Query1RepeatCnt" atomic parts by 
// lookup on their id field.  An index can be used for the
// actual lookup.  
//
// This is the SQL version.
//
/////////////////////////////////////////////////////////////////


extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #1
//
//////////////////////////////////////////////////////////////////


int query1()
{
    int partId;
    QueryIterator *qItr;
    AtomicPart *atomH;
    char theQuery[256];

    // create the iterator once to avoid overhead.
//    qItr = new QueryIterator((char *)0 );

    // set random seed so "hot" runs are truly hot
    srandom(1);

    // now randomly select parts via partId index and process them
    for (int i = 0; i < Query1RepeatCnt; i++) {

	// generate part id and lookup part
	partId = (int) (random() % TotalAtomicParts) + 1;
        sprintf(theQuery, 
                "select A from AllPartsById A where id = %d;",
                partId);
printf("Trying query \"%s\"\n", theQuery);
//        qItr->Reset(theQuery);
        qItr = new QueryIterator(theQuery);
        if (qItr->SQLCode() == QueryFailed) {
            fprintf(stderr, "Query %s failed.\n", theQuery);
            OC_transactionAbort();
            exit(1);
        } else {
            while (qItr->moreData()) {
printf("\tfound a match.\n"); 
                // get next match and advance iterator.
                atomH = (AtomicPart*)(Entity*)(*qItr)();  
                if (debugMode) {
                    printf("    In Query1, partId = %d:\n", partId);
                }
        	// process part by calling the null procedure
                atomH->DoNothing();

            }
        }
        delete qItr;
    }
//    delete qItr;
    return Query1RepeatCnt;
}

/*  $Version: Query1SQL.C 1.1 93/02/19 16:07:17 Exp $ */

