/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <List.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "BenchParams.h"

//////////////////////////////////////////////////////////////////
//
// Query #5 - find all base assemblies B that "use" a composite
// part with a more recent build date than B's build date.
//
// Hand-coded version (no SQL.)
//
/////////////////////////////////////////////////////////////////

extern int	debugMode;


//////////////////////////////////////////////////////////////////
//
// Routine to do Query #5
//
//////////////////////////////////////////////////////////////////


int query5()
{
    DictionaryIterator* modI; // iterator for traversing AllModules collection
                              // (ONTOS object class.)
    Module* modH;  

    SetIterator*	baseI;  // iterator for iterating over all base
			        // assemblies in a module
                                // (ONTOS object class.)

    BaseAssembly* baseH;
    ListIterator* compI;  // iterator for iterating over all composite parts
			// in a base assembly

    CompositePart* compH;

    // scan all modules, scanning all of the base assemblies in each one,
    // scanning all of the composite parts that they use, looking for base
    // assemblies that use more recently dated composite parts

    int count = 0;

    // get new iterator --- defaults to ranging over all dictionaries.
    modI = new DictionaryIterator(AllModulesByName);
    while (modI->moreData()) {
        // get a module
        modH = (Module*)(Entity*) (*modI)();

	// for each module iterate over its base assemblies
	// this assumes that we maintain a collection of base asemblies
	// in each module
    	baseI = new SetIterator(modH->allBases()); 
    	while (baseI->moreData()) {
            baseH = (BaseAssembly*)(Entity*)(*baseI)();
            if (debugMode) {
                printf("In Query5, processing base assembly %d, buildDate = %d\n",
		            baseH->id, baseH->buildDate);
            }
    	    compI = new ListIterator(baseH->componentsPriv()); 
	    while (compI->moreData()) {
                compH = (CompositePart*)(Entity*)(*compI)();
                if (debugMode) {
                    printf("            [Checking composite part %d, buildDate = %d]\n",
		                        compH->id, compH->buildDate);
                }
		if (compH->buildDate > baseH->buildDate) {
	            baseH->DoNothing();
		    count++;
	        }
	    }
	    delete compI;
	}
	delete baseI;
    }
    delete modI;
    return count;
}

/*  $Version: Query5.C 1.1 93/02/19 16:07:28 Exp $ */

