/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <strings.h>
#include <Transaction.h>
#include <stdlib.h>
#include <GlobalEntities.h>
#include "oo7.h"
#include "globals.h"
#include "VarParams.h"


// group to hold all objects.  maybe should become on group for
// each base assembly eventually.

Group *oo7Group;

// Dictionary to hald all base assemblies, indexed by id.
Dictionary *AllBaseAssemblies;

// collection to hold composite parts
Dictionary *AllCompPartsById; 

// collection to hold documents
Dictionary *AllDocumentsByTitle;

// collection to hold documents indexed by ID
Dictionary *AllDocumentsById;

// dictionary to hold all modules 
Dictionary *AllModulesByName;

// Collection of all parts, indexed by ID.
Dictionary *AllPartsById;

// Collection of all parts, indexed by date.
Dictionary *AllPartsByDate;

// global var to point to type object to avoid lookups.
Type *AtomicPartType = NULL;

// global var to point to type object to avoid lookups.
Type *ConnectionType = NULL;

// global var to point to type object to avoid lookups.
Type *CompositePartType = NULL;

// global var to point to type object to avoid lookups.
Type *DocumentType = NULL;

// global var to point to type object to avoid lookups.
Type *ManualType = NULL;

// a set of "dummy" objects for use in flushing the cache.
Set *AllDummyObjects;


// GKK
Type *APType;
Type *CPType;
Type *ASSType;
Type *CAType;
Type *BAType;
Type *CONType;
Type *MANType;
Type *DOCType;
Type *MODType;

////////////////////////////////////////////////////////////////////////
//
// InitGlobals() --- creates and initializes dictionaries and sets
// that manage type extents.  
//
//
////////////////////////////////////////////////////////////////////////
void InitGlobals() {
    int i;

    printf("In InitGlobals, creating oo7Group.\n");
    oo7Group = new Group("oo7Group");

    printf("In InitGlobals, creating AllBaseAssemblies.\n");
    AllBaseAssemblies = new Dictionary((Type *) OC_integer,
                                     // indexed on id
                                     BAType,
                                     FALSE,        // not ordered
                                     FALSE,       // no duplicates.
				     "AllBaseAssemblies"); // pers. name

    printf("In InitGlobals, creating AllPartsById.\n");
    AllPartsById = new Dictionary((Type *) OC_integer,   
                                  APType,
                                  FALSE,           // not ordered
                                  FALSE,           // no duplicates
			          "AllPartsById"); // pers. name

    printf("In InitGlobals, creating AllPartsByDate.\n");
    AllPartsByDate = new Dictionary((Type *) OC_integer,   
                                  APType,
                                  TRUE,             // B+ tree
                                  TRUE,            // could have duplicates
               			  "AllPartsByDate"); // pers. name

    printf("In InitGlobals, creating AllDocumentsByTitle.\n");
    AllDocumentsByTitle = new Dictionary((Type *) OC_string,
                                  DOCType,
                                  TRUE,             // B+ tree
                                  TRUE,            // could have duplicates
  			  	  "AllDocumentsByTitle"); // pers. name

    printf("In InitGlobals, creating AllDocumentsById.\n");
    AllDocumentsById = new Dictionary((Type *) OC_integer,   
                                  DOCType,
                                  FALSE,           // not ordered
                                  FALSE,           // no duplicates
			          "AllDocumentsById"); // pers. name


    printf("In InitGlobals, creating AllCompPartsById.\n");
    AllCompPartsById = new Dictionary((Type *) OC_integer,
                                  CPType,
                                  FALSE,             // not ordered.
                                  FALSE,             // no duplicates
                         	  "AllCompPartsById"); // pers. name

    printf("In InitGlobals, creating AllModulesByName.\n");
    AllModulesByName = new Dictionary((Type *) OC_string,
                                  MODType,
                                  TRUE,             // B+ tree
                                  FALSE,            // could have duplicates
				  "AllModulesByName"); // pers. name

    printf("In InitGlobals, creating AllDummyObjects.\n");

    AllDummyObjects = new (oo7Group, OC_set) Set((Type *) (Type*) OC_lookup("Dummy"));
    printf("Exiting InitGlobals.\n");
}


////////////////////////////////////////////////////////////////////////
//
// EstablishGlobals() --- establishes pointers to all the global objects
//
//
////////////////////////////////////////////////////////////////////////
void EstablishGlobals() 
{
//	printf("establishing pointers to all top level objects\n");

	// establish handle to oo7Group
       	oo7Group = (Group *) OC_lookup("oo7Group");
       	if (oo7Group == NULL) {
		printf("OC_lookup of oo7Group failed\n");
                OC_transactionAbort();
                exit(1);
       	}

	// establish pointers to all the dictionaries 
       	AllBaseAssemblies = (Dictionary *) OC_lookup("AllBaseAssemblies");
       	if (AllBaseAssemblies == NULL) {
		printf("OC_lookup of AllBaseAssemblies failed\n");
                OC_transactionAbort();
                exit(1);
       	}

       	AllCompPartsById = (Dictionary *) OC_lookup("AllCompPartsById");
	if (AllCompPartsById == NULL) {
		printf("OC_lookup of AllCompPartsById failed\n");
                OC_transactionAbort();
                exit(1);
	}

        AllDocumentsByTitle = (Dictionary *) OC_lookup("AllDocumentsByTitle");
	if (AllDocumentsByTitle == NULL) {
		printf("OC_lookup of AllDocumentsByTitle failed\n");
                OC_transactionAbort();
                exit(1);
 	}

        AllDocumentsById = (Dictionary *) OC_lookup("AllDocumentsById");
	if (AllDocumentsById == NULL) {
		printf("OC_lookup of AllDocumentsById failed\n");
                OC_transactionAbort();
                exit(1);
 	}

	AllModulesByName = (Dictionary *) OC_lookup("AllModulesByName");
    	if (AllModulesByName == NULL) {
		printf("OC_lookup of AllModulesByName failed\n");
                OC_transactionAbort();
                exit(1);
    	}

	AllPartsById = (Dictionary *) OC_lookup("AllPartsById");
    	if (AllPartsById == NULL) {
		printf("OC_lookup of AllPartsById failed\n");
                OC_transactionAbort();
                exit(1);
    	}

	AllPartsByDate = (Dictionary *) OC_lookup("AllPartsByDate");
    	if (AllPartsByDate == NULL) {
		printf("OC_lookup of AllPartsByDate failed\n");
                OC_transactionAbort();
                exit(1);
    	}

	AllDummyObjects = (Set *) OC_lookup("Dummy");
    	if (AllDummyObjects == NULL) {
		printf("OC_lookup of Dummy failed\n");
                OC_transactionAbort();
                exit(1);
    	}

	AtomicPartType = (Type*)OC_lookup("AtomicPart");
	CompositePartType = (Type *)OC_lookup("CompositePart");
	ConnectionType = (Type *)OC_lookup("Connection");
	DocumentType = (Type*)OC_lookup("Document");
}

/*  $Version: globals.C 1.4 93/02/25 10:08:51 Exp $ */

